@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # ==============================================================================
 #
-# Copyright 2020-* Emanuele Ballarin <emanuele@ballarin.cc>
+# Copyright 2020-2025 Emanuele Ballarin <emanuele@ballarin.cc>
 # All Rights Reserved. Unless otherwise explicitly stated.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -21,15 +21,13 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 #
-# Versioning
-__version__ = "0.20.0"
-
-# Imports (wildcard)
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 from .data import *
 from .distributed import *
 from .logging import *
 from .nn import *
 from .optim import *
+from .typing import *
 
 # Deletions (from .data)
 del FastCollateMixup
@@ -40,12 +38,12 @@ del data_prep_dispatcher_1ch
 del data_prep_dispatcher_3ch
 del fashionmnist_dataloader_dispatcher
 del imagenette_dataloader_dispatcher
-del mnist_dataloader_dispatcher
 del kmnist_dataloader_dispatcher
-del pathmnist_dataloader_dispatcher
+del mnist_dataloader_dispatcher
 del octmnist_dataloader_dispatcher
-del tissuemnist_dataloader_dispatcher
+del pathmnist_dataloader_dispatcher
 del tinyimagenet_dataloader_dispatcher
+del tissuemnist_dataloader_dispatcher
 
 # Deletions (from .distributed)
 del reduce_accumulate_keepalive
@@ -55,28 +53,31 @@ del slurm_nccl_env
 del ArgMaxLayer
 del BasicAE
 del BasicVAE
-del beta_gaussian_kldiv
-del var_of_lap
 del BinarizeLayer
 del BrokenReLU
 del CausalConv1d
 del Clamp
+del Concatenate
+del ConvNeXtStem
+del ConvStem
 del ConvolutionalFlattenLayer
 del CoordConv1d
 del CoordConv2d
 del CoordConv3d
 del DeepRBL
+del DuplexLinearNeck
 del FCBlock
 del FieldTransform
 del FlatChannelize2DLayer
+del GRNorm
 del GaussianReparameterizerSampler
 del GaussianReparameterizerSamplerLegacy
+del GenerAct
 del InnerProduct
 del KWTA1d
 del KWTA2d
+del MetaAILayerNorm
 del Mish
-del multilasso
-del multiridge
 del MultiSolvePoissonTensor
 del NNEnsemble
 del PoissonNetCifar
@@ -88,40 +89,52 @@ del ReshapeLayer
 del SERLU
 del SGRUHCell
 del ScaledERF
+del SharedDuplexLinearNeck
+del SilhouetteScore
+del SimpleDuplexLinearNeck
+del SimpleFCN
 del SinLU
+del SingleNeckVAE
 del SirenSine
 del SmeLU
 del SolvePoisson
 del SolvePoissonTensor
+del StatefulTupleSelect
 del SwiGLU
+del TeLU
 del TupleDecouple
-del SilhouetteScore
-del Concatenate
-del DuplexLinearNeck
-del SharedDuplexLinearNeck
+del TupleSelect
+del ViTStem
 del WideResNet
+del batched_outer
+del beta_gaussian_kldiv
 del beta_reco_bce
 del beta_reco_bce_splitout
-del lexsemble
+del beta_reco_mse
+del beta_reco_mse_splitout
+del bisided_thresholding
 del build_repeated_sequential
+del cummatmul
 del field_transform
+del lexsemble
+del logit_to_prob
 del mishlayer_init
+del multilasso
+del multiridge
+del oldtranspose
 del pixelwise_bce_mean
 del pixelwise_bce_sum
-del oldtranspose
+del pixelwise_mse_mean
+del pixelwise_mse_sum
+del reco_reg
+del reco_reg_split
 del silhouette_score
-del cummatmul
 del tensor_replicate
-del logit_to_prob
-del bisided_thresholding
-del convstem_block
-del convnext_stem
-del smallconv_featurizer
-del MetaAILayerNorm
+del var_of_lap
 
 # Deletions (from .optim)
+del ADOPT
 del AdaBound
-del AdaHessian
 del AdamP
 del Adan
 del CosineLRScheduler
@@ -129,19 +142,32 @@ del Lamb
 del Lion
 del Lookahead
 del Lookaround
+del MultiPhaseScheduler
 del RAdam
 del SAM
 del SGDP
+del alah_optim
 del epochwise_onecycle
+del lahdopt
+del make_beta_scheduler
 del onecycle_lincos
 del onecycle_linlin
 del onecycle_linlin_updown
-del warmed_up_linneal
-del make_beta_scheduler
 del ralah_optim
 del tricyc1c
+del warmed_up_annealer
+del warmed_up_linneal
 del wfneal
 
 # Deletions (from .logging)
 del AverageMeter
+del MultiAverageMeter
 del LogCSV
+del write_dict_as_yaml
+
+# Deletions (from .typing)
+del actvt
+del numlike
+del realnum
+del strdev
+del tensorlike
