@@ -1,23 +1,24 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# ------------------------------------------------------------------------------
+# ──────────────────────────────────────────────────────────────────────────────
 #
-#  Copyright (c) 2020-2023 Emanuele Ballarin <emanuele@ballarin.cc>
+#  Copyright (c) 2020-2025 Emanuele Ballarin <emanuele@ballarin.cc>
 #  Released under the terms of the MIT License
 #  (see: https://url.ballarin.cc/mitlicense)
 #
-# ------------------------------------------------------------------------------
+# ──────────────────────────────────────────────────────────────────────────────
 #
 # SPDX-License-Identifier: MIT
 #
-# ------------------------------------------------------------------------------
+# ──────────────────────────────────────────────────────────────────────────────
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import os
-from typing import Tuple
 
-__all__ = ["slurm_nccl_env"]
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ["slurm_nccl_env"]
 
 
-def slurm_nccl_env() -> Tuple[int, int, int, int, int, str]:
+def slurm_nccl_env() -> tuple[int, int, int, int, int, str]:
     """
     Get SLURM environment variables for NCCL-based distributed training.
     Remember to define in the SLURM script the following environment variable(s):
