@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # ──────────────────────────────────────────────────────────────────────────────
 #
-#  Copyright (c) 2020-2024 Emanuele Ballarin <emanuele@ballarin.cc>
+#  Copyright (c) 2020-2025 Emanuele Ballarin <emanuele@ballarin.cc>
 #  Released under the terms of the MIT License
 #  (see: https://url.ballarin.cc/mitlicense)
 #
@@ -11,10 +11,13 @@
 # SPDX-License-Identifier: MIT
 #
 # ──────────────────────────────────────────────────────────────────────────────
-# Imports (specific)
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 from .avgmeter import AverageMeter
+from .avgmeter import MultiAverageMeter
 from .logcsv import LogCSV
+from .yamldump import write_dict_as_yaml
 
-# Deletions (from .)
+# ──────────────────────────────────────────────────────────────────────────────
 del avgmeter
 del logcsv
+del yamldump
