@@ -2,8 +2,8 @@
 # -*- coding: utf-8 -*-
 # ==============================================================================
 #
-# Copyright (c) 2020-* Emanuele Ballarin <emanuele@ballarin.cc>
-#                      All Rights Reserved.
+# Copyright (c) 2020-2025 Emanuele Ballarin <emanuele@ballarin.cc>
+#                         All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,14 +19,11 @@
 #
 # ==============================================================================
 # SPDX-License-Identifier: Apache-2.0
-# IMPORTS
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import copy
 from collections.abc import Callable
 from math import copysign
-from typing import List
-from typing import Optional
-from typing import Tuple
-from typing import Union
+from math import sqrt as msqrt
 
 import torch
 from torch import nn
@@ -34,14 +31,21 @@ from torch import Tensor
 from torch.nn import functional as F
 from torch.utils.hooks import RemovableHandle
 
+from ..typing import realnum
 from .functional import silhouette_score
 from .penalties import beta_gaussian_kldiv
+from .utils import fxfx2module
 
-__all__ = [
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = [
     "pixelwise_bce_sum",
     "pixelwise_bce_mean",
     "beta_reco_bce",
     "beta_reco_bce_splitout",
+    "pixelwise_mse_sum",
+    "pixelwise_mse_mean",
+    "beta_reco_mse",
+    "beta_reco_mse_splitout",
     "FCBlock",
     "CausalConv1d",
     "build_repeated_sequential",
@@ -56,27 +60,28 @@ __all__ = [
     "SirenSine",
     "BasicAE",
     "BasicVAE",
+    "SingleNeckVAE",
     "Clamp",
     "SwiGLU",
     "TupleDecouple",
+    "TupleSelect",
+    "StatefulTupleSelect",
     "SilhouetteScore",
     "Concatenate",
     "DuplexLinearNeck",
     "SharedDuplexLinearNeck",
+    "SimpleDuplexLinearNeck",
     "GaussianReparameterizerSamplerLegacy",
     "lexsemble",
+    "GenerAct",
+    "SimpleFCN",
 ]
 
-# CUSTOM TYPES
-realnum = Union[float, int]
-
 
 # Ensembling functions
 @torch.jit.script
 def lexsemble(x: Tensor, cls_dim: int = -2, ens_dim: int = -1) -> Tensor:
-    out: Tensor = torch.softmax(
-        torch.exp(x).sum(dim=ens_dim), dim=cls_dim - int(copysign(1, cls_dim))
-    )
+    out: Tensor = torch.softmax(torch.exp(x).sum(dim=ens_dim), dim=cls_dim - int(copysign(1, cls_dim)))
     return torch.log(out / (1 - out))
 
 
@@ -86,11 +91,21 @@ def pixelwise_bce_sum(lhs: Tensor, rhs: Tensor) -> Tensor:
     return F.binary_cross_entropy(lhs, rhs, reduction="sum")
 
 
+@torch.jit.script
+def pixelwise_mse_sum(lhs: Tensor, rhs: Tensor) -> Tensor:
+    return F.mse_loss(lhs, rhs, reduction="sum")
+
+
 @torch.jit.script
 def pixelwise_bce_mean(lhs: Tensor, rhs: Tensor) -> Tensor:
     return F.binary_cross_entropy(lhs, rhs, reduction="mean")
 
 
+@torch.jit.script
+def pixelwise_mse_mean(lhs: Tensor, rhs: Tensor) -> Tensor:
+    return F.mse_loss(lhs, rhs, reduction="mean")
+
+
 @torch.jit.script
 def beta_reco_bce(
     input_reco: Tensor,
@@ -104,6 +119,19 @@ def beta_reco_bce(
     return pwbce + kldiv
 
 
+@torch.jit.script
+def beta_reco_mse(
+    input_reco: Tensor,
+    input_orig: Tensor,
+    mu: Tensor,
+    sigma: Tensor,
+    beta: float = 1.0,
+):
+    kldiv = beta_gaussian_kldiv(mu, sigma, beta)
+    pwmse = pixelwise_mse_sum(input_reco, input_orig)
+    return pwmse + kldiv
+
+
 @torch.jit.script
 def beta_reco_bce_splitout(
     input_reco: Tensor,
@@ -117,10 +145,21 @@ def beta_reco_bce_splitout(
     return pwbce + kldiv, pwbce, kldiv
 
 
+@torch.jit.script
+def beta_reco_mse_splitout(
+    input_reco: Tensor,
+    input_orig: Tensor,
+    mu: Tensor,
+    sigma: Tensor,
+    beta: float = 1.0,
+):
+    kldiv = beta_gaussian_kldiv(mu, sigma, beta)
+    pwmse = pixelwise_mse_sum(input_reco, input_orig)
+    return pwmse + kldiv, pwmse, kldiv
+
+
 # Utility functions
-def _rbm_mask_generator(
-    size: int, ood_width: int = 0, dens: float = 1.0, rand_diag: bool = False
-) -> torch.Tensor:
+def _rbm_mask_generator(size: int, ood_width: int = 0, dens: float = 1.0, rand_diag: bool = False) -> torch.Tensor:
     """
     Generate a random band matrix mask.
     """
@@ -137,23 +176,25 @@ def _rbm_mask_generator(
     for i in range(ood_width):
         offset: int = i + 1
         mask: Tensor = torch.logical_or(
-            mask, torch.diag(torch.rand(size - offset) <= dens, diagonal=offset)  # type: ignore
+            mask,
+            torch.diag(torch.rand(size - offset) <= dens, diagonal=offset),  # type: ignore
         )
         mask: Tensor = torch.logical_or(
-            mask, torch.diag(torch.rand(size - offset) <= dens, diagonal=-offset)  # type: ignore
+            mask,
+            torch.diag(torch.rand(size - offset) <= dens, diagonal=-offset),  # type: ignore
         )
 
     return mask
 
 
 def _masked_gradient_hook_factory(
-    mask: Union[Tensor, None]
-) -> Callable[[Union[Tensor, None]], Union[Tensor, None]]:
+    mask: Tensor | None,
+) -> Callable[[Tensor | None], Tensor | None]:
     """
     Return a backward hook that masks the gradient with the given mask.
     """
 
-    def _masked_gradient_hook(grad: Union[Tensor, None]) -> Union[Tensor, None]:
+    def _masked_gradient_hook(grad: Tensor | None) -> Tensor | None:
         """
         Backward hook that masks the gradient with nonlocal variable `mask`.
         """
@@ -162,18 +203,14 @@ def _masked_gradient_hook_factory(
             return grad
 
         if grad.shape != mask.shape:
-            raise ValueError(
-                f"Gradient shape {grad.shape} is not equal to mask shape {mask.shape}"
-            )
+            raise ValueError(f"Gradient shape {grad.shape} is not equal to mask shape {mask.shape}")
 
         return grad * mask.to(grad.device)
 
     return _masked_gradient_hook
 
 
-def build_repeated_sequential(
-    depth: int, rep_builder: Callable[[int], nn.Module]
-) -> nn.Sequential:
+def build_repeated_sequential(depth: int, rep_builder: Callable[[int], nn.Module]) -> nn.Sequential:
     repeated: nn.Sequential = nn.Sequential()
     i: int
     for i in range(depth):
@@ -181,25 +218,55 @@ def build_repeated_sequential(
     return repeated
 
 
+class SimpleFCN(nn.Module):
+    def __init__(
+        self,
+        in_dim: int,
+        out_dim: int,
+        hid_dims: list[int] | tuple[int, ...] | int | None = None,
+        actv: nn.Module | Callable[[Tensor], Tensor] = nn.Mish(),
+        final_actv: nn.Module | Callable[[Tensor], Tensor] | None = None,
+        normalize: bool = False,
+    ) -> None:
+        super().__init__()
+        if hid_dims is None:
+            hid_dims: list[int] | tuple[int, ...] = []
+        elif not isinstance(hid_dims, (list, tuple)):
+            hid_dims: list[int] | tuple[int, ...] = (hid_dims,)
+        layers: list[nn.Module] = []
+        prev_dim: int = in_dim
+        for dim in hid_dims:
+            layers.append(nn.Linear(prev_dim, dim))
+            layers.append(copy.deepcopy(actv))
+            if normalize and (dim > 1):
+                layers.append(nn.LayerNorm(dim))
+            prev_dim: int = dim
+        layers.append(nn.Linear(prev_dim, out_dim))
+        if final_actv is not None:
+            layers.append(copy.deepcopy(final_actv))
+        self.net = nn.Sequential(*layers)
+
+    def forward(self, x: Tensor) -> Tensor:
+        return self.net(x)
+
+
 # Fully-Connected Block, New version
 # Joint work with Davide Roznowicz (https://github.com/DavideRoznowicz)
 class FCBlock(nn.Module):
-    def __init__(
+    def __init__(  # NOSONAR
         self,
-        in_sizes: Union[List[int], tuple],
+        in_sizes: list[int] | tuple,
         out_size: int,
-        bias: Optional[Union[List[bool], tuple, bool]] = None,
-        activation_fx: Optional[Union[List, nn.ModuleList, nn.Module]] = None,
-        dropout: Optional[Union[List[Union[float, bool]], float, bool, tuple]] = None,
-        batchnorm: Optional[Union[List[bool], bool, tuple]] = None,
+        bias: list[bool] | tuple | bool | None = None,
+        activation_fx: list | nn.ModuleList | nn.Module | None = None,
+        dropout: list[float | bool] | float | bool | tuple | None = None,
+        batchnorm: list[bool] | bool | tuple | None = None,
     ) -> None:
         super().__init__()
 
         self.activation_fx = nn.ModuleList()
 
-        error_uneven_size: str = (
-            "The length of lists of arguments must be the same across them."
-        )
+        error_uneven_size: str = "The length of lists of arguments must be the same across them."
         error_illegal_dropout: str = (
             "The 'dropout' argument must be either False, a float, or an iterable of floats and/or False."
         )
@@ -224,9 +291,7 @@ class FCBlock(nn.Module):
                 dropout = [False] * len(in_sizes)
             else:
                 raise ValueError(error_illegal_dropout)
-        elif isinstance(dropout, float) or (
-            isinstance(dropout, int) and (dropout in (0, 1))
-        ):
+        elif isinstance(dropout, float) or (isinstance(dropout, int) and (dropout in (0, 1))):
             dropout = [dropout] * len(in_sizes)
         elif not isinstance(dropout, list):
             raise ValueError(error_illegal_dropout)
@@ -236,22 +301,14 @@ class FCBlock(nn.Module):
 
         if isinstance(activation_fx, list):
             self.activation_fx = nn.ModuleList(copy.deepcopy(activation_fx))
-        elif isinstance(activation_fx, nn.Module) and not isinstance(
-            activation_fx, nn.ModuleList
-        ):
+        elif isinstance(activation_fx, nn.Module) and not isinstance(activation_fx, nn.ModuleList):
             for _ in enumerate(in_sizes):
                 self.activation_fx.append(copy.deepcopy(activation_fx))
         elif isinstance(activation_fx, nn.ModuleList):
             self.activation_fx = copy.deepcopy(activation_fx)
 
         # Sanitize
-        if (
-            not len(in_sizes)
-            == len(bias)
-            == len(self.activation_fx)
-            == len(dropout)
-            == len(batchnorm)
-        ):
+        if not len(in_sizes) == len(bias) == len(self.activation_fx) == len(dropout) == len(batchnorm):
             raise ValueError(error_uneven_size)
 
         # Start with an empty module list
@@ -268,18 +325,14 @@ class FCBlock(nn.Module):
             self.module_battery.append(copy.deepcopy(self.activation_fx[layer_idx]))
 
             if batchnorm[layer_idx]:
-                self.module_battery.append(
-                    nn.BatchNorm1d(num_features=in_sizes[layer_idx + 1])
-                )
+                self.module_battery.append(nn.BatchNorm1d(num_features=in_sizes[layer_idx + 1]))
 
             if isinstance(dropout[layer_idx], bool) and dropout[layer_idx]:
                 raise ValueError(error_illegal_dropout)
             if not isinstance(dropout[layer_idx], bool):
                 self.module_battery.append(nn.Dropout(p=dropout[layer_idx]))
 
-        self.module_battery.append(
-            nn.Linear(in_features=in_sizes[-1], out_features=out_size, bias=bias[-1])
-        )
+        self.module_battery.append(nn.Linear(in_features=in_sizes[-1], out_features=out_size, bias=bias[-1]))
         self.module_battery.append(copy.deepcopy(self.activation_fx[-1]))
         if batchnorm[-1]:
             self.module_battery.append(nn.BatchNorm1d(num_features=out_size))
@@ -305,42 +358,31 @@ class FCBlockLegacy(nn.Module):
     def __init__(
         self,
         fin: int,
-        hsizes: List[int],
+        hsizes: list[int],
         fout: int,
         hactiv,
         oactiv,
-        bias: Union[bool, List[bool]] = True,
+        bias: bool | list[bool] = True,
     ) -> None:
         super().__init__()
-        allsizes: List[int] = [fin] + hsizes + [fout]
+        allsizes: list[int] = [fin] + hsizes + [fout]
 
         # Biases for the linears below
         if not isinstance(bias, list):
             bias = [bias] * (len(allsizes) - 1)
         else:
             if len(bias) != len(allsizes) - 1:
-                raise RuntimeError(
-                    "If 'bias' is a list, it must have as many elements as #linears"
-                )
+                raise RuntimeError("If 'bias' is a list, it must have as many elements as #linears")
 
-        self.linears: nn.ModuleList = nn.ModuleList(
-            [
-                nn.Linear(allsizes[i], allsizes[i + 1], bias=bias[i])
-                for i in range(0, len(allsizes) - 1)
-            ]
-        )
+        self.linears: nn.ModuleList = nn.ModuleList([
+            nn.Linear(allsizes[i], allsizes[i + 1], bias=bias[i]) for i in range(0, len(allsizes) - 1)
+        ])
         self.hactiv = hactiv
         self.oactiv = oactiv
 
         # Address the "hactiv: list" case
-        if (
-            hactiv is not None
-            and isinstance(hactiv, list)
-            and (len(hactiv) != len(self.linears) - 1)
-        ):
-            raise RuntimeError(
-                "If 'hactiv' is a list, it must have as many elements as (#linears - 1)"
-            )
+        if hactiv is not None and isinstance(hactiv, list) and (len(hactiv) != len(self.linears) - 1):
+            raise RuntimeError("If 'hactiv' is a list, it must have as many elements as (#linears - 1)")
 
     def forward(self, x: Tensor) -> Tensor:
         idx: int
@@ -359,8 +401,6 @@ class FCBlockLegacy(nn.Module):
 
 # Causal Convolutional Layer, 1D
 # (cfr.: https://github.com/pytorch/pytorch/issues/1333#issuecomment-400338207)
-
-
 class CausalConv1d(nn.Conv1d):
     def __init__(
         self,
@@ -395,17 +435,11 @@ class CausalConv1d(nn.Conv1d):
 
 
 # Reparameterizer / Sampler for (C)VAEs & co.
-
-
-def _gauss_reparameterize_sample(
-    z_mu: Tensor, z_log_var: Tensor, device: Optional[torch.DeviceObjType] = None
-) -> Tensor:
+def _gauss_reparameterize_sample(z_mu: Tensor, z_log_var: Tensor, device: torch.DeviceObjType | None = None) -> Tensor:
     if device is None:
         device = z_mu.device
         if device != z_log_var.device:
-            raise RuntimeError(
-                f"Device mismatch among 'z_mu' ({device}) and 'z_log_var' ({z_log_var.device})!"
-            )
+            raise RuntimeError(f"Device mismatch among 'z_mu' ({device}) and 'z_log_var' ({z_log_var.device})!")
     return z_mu.to(device) + torch.randn_like(z_mu).to(device) * torch.exp(
         z_log_var * 0.5  # type: ignore
     ).to(device)
@@ -426,9 +460,7 @@ class GaussianReparameterizerSampler(nn.Module):
 
     # noinspection PyMethodMayBeStatic
     def forward(self, z_mu: torch.Tensor, z_log_var: torch.Tensor) -> torch.Tensor:
-        return z_mu + torch.randn_like(z_mu, device=z_mu.device) * torch.exp(
-            z_log_var * 0.5
-        )
+        return z_mu + torch.randn_like(z_mu, device=z_mu.device) * torch.exp(z_log_var * 0.5)
 
 
 class SGRUHCell(nn.Module):
@@ -446,21 +478,16 @@ class SGRUHCell(nn.Module):
         batch_first: bool = False,
         dropout: realnum = 0,
         bidirectional: bool = False,
-        tbptt: Union[bool, int] = False,
-        hx: Optional[Tensor] = None,
+        tbptt: bool | int = False,
+        hx: Tensor | None = None,
         readin_head: nn.Module = nn.Identity(),
         readout_head: nn.Module = nn.Identity(),
     ) -> None:
         super().__init__()
 
         # Validate tbptt (otherwise it could become impossible to catch mistakes!)
-        if not (
-            (isinstance(tbptt, bool) and not tbptt)
-            or (isinstance(tbptt, int) and tbptt >= 0)
-        ):
-            raise ValueError(
-                f"Parameter 'tbptt' must be either False or a positive integer. Given: {tbptt}"
-            )
+        if not ((isinstance(tbptt, bool) and not tbptt) or (isinstance(tbptt, int) and tbptt >= 0)):
+            raise ValueError(f"Parameter 'tbptt' must be either False or a positive integer. Given: {tbptt}")
         self._tbptt: int = int(tbptt)  # False == 0
 
         # Copy and store read-heads
@@ -482,18 +509,14 @@ class SGRUHCell(nn.Module):
         if hx is not None:
             self._hx: Tensor = hx.clone()
         else:
-            self._hx = hx
+            self._hx = hx  # type: ignore
 
         # Track recurrence
         self._recurrence_idx: int = 0
 
     def forward(self, x: Tensor) -> Tensor:
         # Implement Truncated BPTT (if requested to do so)
-        if (
-            self._tbptt > 0
-            and self._recurrence_idx > 0
-            and self._recurrence_idx % self._tbptt == 0
-        ):
+        if self._tbptt > 0 and self._recurrence_idx > 0 and self._recurrence_idx % self._tbptt == 0:
             self._hx: Tensor = self._hx.detach()
 
         # Read input in, through the readin head
@@ -515,18 +538,18 @@ class SGRUHCell(nn.Module):
         # Return
         return out
 
-    def reset_hx(self, hx: Optional[Tensor] = None) -> None:
+    def reset_hx(self, hx: Tensor | None = None) -> None:
         # Register the detachment of self._hx in PyTorch computational graph
         if self._hx is not None:
             self._hx: Tensor = self._hx.detach()
         else:
-            self._hx = None
+            self._hx = None  # type: ignore
 
         # Re-initialize self._hx
         if hx is not None:
             self._hx: Tensor = hx.clone()
         else:
-            self._hx = hx
+            self._hx = hx  # type: ignore
 
         # Restart the recurrence index
         self._recurrence_idx: int = 0
@@ -594,7 +617,7 @@ class RBLinear(nn.Linear):
 
     features: int
     w_mask: Tensor
-    b_mask: Union[Tensor, None]
+    b_mask: Tensor | None
     ood_width: int
     dens: float
     rand_diag: bool
@@ -629,22 +652,26 @@ class RBLinear(nn.Linear):
         self.rand_bias = rand_bias
 
         # Instantiate masks
-        _w_mask: Tensor = _rbm_mask_generator(
-            size=features, ood_width=ood_width, dens=dens, rand_diag=rand_diag
-        ).to(self.weight.device)
+        _w_mask: Tensor = _rbm_mask_generator(size=features, ood_width=ood_width, dens=dens, rand_diag=rand_diag).to(
+            self.weight.device
+        )
         self.register_buffer(name="w_mask", tensor=_w_mask, persistent=True)
 
         if self.bias is not None:
             if rand_bias:
-                _b_mask: Tensor = (torch.rand_like(self.bias) <= dens).to(self.bias.device)  # type: ignore
+                _b_mask: Tensor = (torch.rand_like(self.bias) <= dens).to(  # type: ignore
+                    self.bias.device  # type: ignore
+                )  # type: ignore
             else:
-                _b_mask: Tensor = torch.ones_like(self.bias, dtype=torch.bool).to(self.bias.device)  # type: ignore
+                _b_mask: Tensor = torch.ones_like(self.bias, dtype=torch.bool).to(  # type: ignore
+                    self.bias.device  # type: ignore
+                )  # type: ignore
         else:
             _b_mask = None  # type: ignore
         self.register_buffer(name="b_mask", tensor=_b_mask, persistent=True)
 
         # Mask and hook weight and bias
-        self.hook_handles: List[RemovableHandle] = []
+        self.hook_handles: list[RemovableHandle] = []
         self._mask_and_hook()
 
     def _mask_and_hook(self) -> None:
@@ -659,9 +686,7 @@ class RBLinear(nn.Linear):
         w_handle = self.weight.register_hook(_masked_gradient_hook_factory(self.w_mask))
         self.hook_handles.append(w_handle)
         if self.bias is not None:
-            b_handle = self.bias.register_hook(
-                _masked_gradient_hook_factory(self.b_mask)
-            )
+            b_handle = self.bias.register_hook(_masked_gradient_hook_factory(self.b_mask))
             self.hook_handles.append(b_handle)
 
     def _reset_parameters(self) -> None:
@@ -699,8 +724,8 @@ class DeepRBL(nn.Module):
         self,
         features: int,
         depth: int = 1,
-        act_fx: Union[nn.Module, Callable] = nn.Identity(),
-        act_final_fx: Optional[Union[nn.Module, Callable]] = None,
+        act_fx: nn.Module | Callable = nn.Identity(),
+        act_final_fx: nn.Module | Callable | None = None,
         batchnorm: bool = False,
         ood_width: int = 0,
         dens: float = 1.0,
@@ -725,8 +750,8 @@ class DeepRBL(nn.Module):
         # Store arguments
         self.features: int = features
         self.depth: int = depth
-        self.act_fx: Union[nn.Module, Callable] = act_fx
-        self.act_final_fx: Union[nn.Module, Callable] = act_final_fx
+        self.act_fx: nn.Module | Callable = act_fx
+        self.act_final_fx: nn.Module | Callable = act_final_fx
         self.batchnorm: bool = batchnorm
         self.ood_width: int = ood_width
         self.dens: float = dens
@@ -785,17 +810,19 @@ class DeepRBL(nn.Module):
 class ResBlock(nn.Module):
     def __init__(
         self,
-        block: Union[nn.Module, Callable[[Tensor], Tensor]],
-        shortcut: Union[nn.Module, Callable[[Tensor], Tensor]] = nn.Identity(),
-        postall: Union[nn.Module, Callable[[Tensor], Tensor]] = nn.Identity(),
+        block: nn.Module | Callable[[Tensor], Tensor],
+        shortcut: nn.Module | Callable[[Tensor], Tensor] = nn.Identity(),
+        postall: nn.Module | Callable[[Tensor], Tensor] = nn.Identity(),
+        sqrt2norm: bool = False,
     ) -> None:
         super().__init__()
-        self.block: Union[nn.Module, Callable[[Tensor], Tensor]] = block
-        self.shtct: Union[nn.Module, Callable[[Tensor], Tensor]] = shortcut
-        self.postl: Union[nn.Module, Callable[[Tensor], Tensor]] = postall
+        self.block: nn.Module | Callable[[Tensor], Tensor] = block
+        self.shtct: nn.Module | Callable[[Tensor], Tensor] = shortcut
+        self.postl: nn.Module | Callable[[Tensor], Tensor] = postall
+        self.addnorm: realnum = msqrt(2) if sqrt2norm else 1
 
     def forward(self, x: Tensor) -> Tensor:
-        return self.postl(self.shtct(x) + self.block(x))
+        return self.postl((self.shtct(x) + self.block(x)) / self.addnorm)
 
 
 # SIREN-like Sine activation(s)
@@ -805,7 +832,7 @@ class SirenSine(nn.Module):
         if learn_w0:
             self.w0: nn.Parameter = nn.Parameter(torch.tensor([w0]))
         else:
-            self.w0: float = w0
+            self.w0: float = w0  # type: ignore
 
     def forward(self, x: Tensor) -> Tensor:
         return torch.sin(self.w0 * x)
@@ -824,7 +851,7 @@ class BasicAE(nn.Module):
         self.decoder: nn.Module = decoder
         self.extract_z: bool = extract_z
 
-    def forward(self, x: Tensor) -> Union[Tensor, Tuple[Tensor, Tensor]]:
+    def forward(self, x: Tensor) -> Tensor | tuple[Tensor, Tensor]:
         z: Tensor = self.encoder(x)
         y: Tensor = self.decoder(z)
         if self.extract_z:
@@ -852,15 +879,49 @@ class BasicVAE(nn.Module):
         self.extract_z: bool = extract_z
         self.extract_mv: bool = extract_mv
 
-    def forward(self, x: Tensor) -> Union[
-        Tensor,
-        Tuple[Tensor, Tensor],
-        Tuple[Tensor, Tensor, Tensor],
-        Tuple[Tensor, Tensor, Tensor, Tensor],
-    ]:
+    def forward(
+        self, x: Tensor
+    ) -> Tensor | tuple[Tensor, Tensor] | tuple[Tensor, Tensor, Tensor] | tuple[Tensor, Tensor, Tensor, Tensor]:
         shared: Tensor = self.encoder(x)
         mean: Tensor = self.mean_neck(shared)
         logvar: Tensor = self.logvar_neck(shared)
+        # noinspection DuplicatedCode
+        z: Tensor = self.grps(mean, logvar)
+        y: Tensor = self.decoder(z)
+
+        if self.extract_z and self.extract_mv:
+            return y, z, mean, logvar
+        elif self.extract_z:
+            return y, z
+        elif self.extract_mv:
+            return y, mean, logvar
+        else:
+            return y
+
+
+class SingleNeckVAE(nn.Module):
+    def __init__(
+        self,
+        encoder: nn.Module,
+        neck: nn.Module,
+        decoder: nn.Module,
+        extract_z: bool = False,
+        extract_mv: bool = False,
+    ) -> None:
+        super().__init__()
+        self.encoder: nn.Module = encoder
+        self.neck: nn.Module = neck
+        self.grps: GaussianReparameterizerSampler = GaussianReparameterizerSampler()
+        self.decoder: nn.Module = decoder
+        self.extract_z: bool = extract_z
+        self.extract_mv: bool = extract_mv
+
+    def forward(
+        self, x: Tensor
+    ) -> Tensor | tuple[Tensor, Tensor] | tuple[Tensor, Tensor, Tensor] | tuple[Tensor, Tensor, Tensor, Tensor]:
+        shared: Tensor = self.encoder(x)
+        mean, logvar = self.neck(shared)
+        # noinspection DuplicatedCode
         z: Tensor = self.grps(mean, logvar)
         y: Tensor = self.decoder(z)
 
@@ -897,7 +958,7 @@ class TupleDecouple(nn.Module):
         self.module: nn.Module = module
         self.idx: int = idx
 
-    def forward(self, xtuple: Tuple[Tensor, ...]) -> Tuple[Tensor, ...]:
+    def forward(self, xtuple: tuple[Tensor, ...]) -> tuple[Tensor, ...]:
         return (
             *xtuple[: self.idx],
             self.module(xtuple[self.idx]),
@@ -905,6 +966,25 @@ class TupleDecouple(nn.Module):
         )
 
 
+class TupleSelect(nn.Module):
+    def __init__(self, idx: int = 0) -> None:
+        super().__init__()
+        self.idx: int = idx
+
+    def forward(self, xtuple: tuple[Tensor, ...]) -> Tensor:
+        return xtuple[self.idx]
+
+
+class StatefulTupleSelect(nn.Module):
+    def __init__(self, module, idx: int = 0) -> None:
+        super().__init__()
+        self.module: nn.Module = module
+        self.idx: int = idx
+
+    def forward(self, x: Tensor) -> Tensor:
+        return self.module(x)[self.idx]
+
+
 class SilhouetteScore(nn.Module):
     """
     Layerized computation of the Silhouette Score.
@@ -921,12 +1001,8 @@ class Concatenate(nn.Module):
         self.dim: int = dim
         self.flatten: bool = flatten
 
-    def forward(self, tensors: Union[Tuple[torch.Tensor, ...], List[torch.Tensor]]):
-        tensors = (
-            [tensor.flatten(start_dim=1) for tensor in tensors]
-            if self.flatten
-            else tensors
-        )
+    def forward(self, tensors: tuple[torch.Tensor, ...] | list[torch.Tensor]):
+        tensors = [tensor.flatten(start_dim=1) for tensor in tensors] if self.flatten else tensors
         return torch.cat(tensors, dim=self.dim)
 
 
@@ -936,9 +1012,7 @@ class DuplexLinearNeck(nn.Module):
         self.x_to_mu: nn.Linear = nn.Linear(in_dim, latent_dim)
         self.x_to_log_var: nn.Linear = nn.Linear(in_dim, latent_dim)
 
-    def forward(
-        self, xc: Union[Tuple[torch.Tensor, ...], List[torch.Tensor]]
-    ) -> Tuple[torch.Tensor, torch.Tensor]:
+    def forward(self, xc: tuple[torch.Tensor, ...] | list[torch.Tensor]) -> tuple[torch.Tensor, torch.Tensor]:
         cxc: torch.Tensor = torch.cat(xc, dim=1)
         return self.x_to_mu(cxc), self.x_to_log_var(cxc)
 
@@ -948,9 +1022,39 @@ class SharedDuplexLinearNeck(nn.Module):
         super().__init__()
         self.shared_layer: nn.Linear = nn.Linear(in_dim, 2 * latent_dim)
 
-    def forward(
-        self, xc: Union[Tuple[torch.Tensor, ...], List[torch.Tensor]]
-    ) -> Tuple[torch.Tensor, torch.Tensor]:
+    def forward(self, xc: tuple[torch.Tensor, ...] | list[torch.Tensor]) -> tuple[torch.Tensor, torch.Tensor]:
         cxc: torch.Tensor = torch.cat(xc, dim=1)
         # noinspection PyTypeChecker
         return torch.chunk(self.shared_layer(cxc), 2, dim=1)
+
+
+class SimpleDuplexLinearNeck(nn.Module):
+    def __init__(self, in_dim: int, latent_dim: int):
+        super().__init__()
+        self.x_to_mu: nn.Linear = nn.Linear(in_dim, latent_dim)
+        self.x_to_log_var: nn.Linear = nn.Linear(in_dim, latent_dim)
+
+    def forward(self, x: Tensor) -> tuple[Tensor, Tensor]:
+        return self.x_to_mu(x), self.x_to_log_var(x)
+
+
+class GenerAct(nn.Module):
+    def __init__(
+        self,
+        act: Callable[[Tensor], Tensor] | nn.Module,
+        subv: float | None = None,
+        maxv: float | None = None,
+        minv: float | None = None,
+    ):
+        super().__init__()
+        self.act: nn.Module = fxfx2module(act)
+        self.subv: float | None = subv
+        self.maxv: float | None = maxv
+        self.minv: float | None = minv
+
+    def forward(self, x: Tensor) -> Tensor:
+        x: Tensor = self.act(x)
+        x: Tensor = x - self.subv if self.subv is not None else x
+        x: Tensor = x.clamp_max(self.maxv) if self.maxv is not None else x
+        x: Tensor = x.clamp_min(self.minv) if self.minv is not None else x
+        return x
