@@ -3,7 +3,7 @@
 # ──────────────────────────────────────────────────────────────────────────────
 # Copyright 2020 Deepmind Technologies Limited.
 # Copyright 2023 Cui Jiequan (崔洁全). Minor edits.
-# Copyright 2023 Emanuele Ballarin <emanuele@ballarin.cc>. Minor edits.
+# Copyright 2025 Emanuele Ballarin <emanuele@ballarin.cc>. Minor edits.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,18 +19,16 @@
 #
 # ──────────────────────────────────────────────────────────────────────────────
 """WideResNet and PreActResNet implementations in PyTorch. From DeepMind's original."""
-from typing import List
-from typing import Optional
-from typing import Tuple
-from typing import Union
 
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
 
 # ──────────────────────────────────────────────────────────────────────────────
 
-__all__ = [
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = [
     "WideResNet",
     "PreActResNet",
     "CIFAR10_MEAN",
@@ -45,14 +43,14 @@ __all__ = [
 
 # ──────────────────────────────────────────────────────────────────────────────
 
-CIFAR10_MEAN: Tuple[float, float, float] = (0.4914, 0.4822, 0.4465)
-CIFAR10_STD: Tuple[float, float, float] = (0.2471, 0.2435, 0.2616)
-CIFAR100_MEAN: Tuple[float, float, float] = (0.5071, 0.4865, 0.4409)
-CIFAR100_STD: Tuple[float, float, float] = (0.2673, 0.2564, 0.2762)
-SVHN_MEAN: Tuple[float, float, float] = (0.5, 0.5, 0.5)
-SVHN_STD: Tuple[float, float, float] = (0.5, 0.5, 0.5)
-TINY_MEAN: Tuple[float, float, float] = (0.4802, 0.4481, 0.3975)
-TINY_STD: Tuple[float, float, float] = (0.2302, 0.2265, 0.2262)
+CIFAR10_MEAN: tuple[float, float, float] = (0.4914, 0.4822, 0.4465)
+CIFAR10_STD: tuple[float, float, float] = (0.2471, 0.2435, 0.2616)
+CIFAR100_MEAN: tuple[float, float, float] = (0.5071, 0.4865, 0.4409)
+CIFAR100_STD: tuple[float, float, float] = (0.2673, 0.2564, 0.2762)
+SVHN_MEAN: tuple[float, float, float] = (0.5, 0.5, 0.5)
+SVHN_STD: tuple[float, float, float] = (0.5, 0.5, 0.5)
+TINY_MEAN: tuple[float, float, float] = (0.4802, 0.4481, 0.3975)
+TINY_STD: tuple[float, float, float] = (0.2302, 0.2265, 0.2262)
 
 # ──────────────────────────────────────────────────────────────────────────────
 
@@ -93,17 +91,13 @@ class _Block(nn.Module):
         super().__init__()
         self.batchnorm_0: nn.Module = nn.BatchNorm2d(in_planes, momentum=bn_momentum)
         self.relu_0: nn.Module = activation_fn()
-        self.conv_0: nn.Module = nn.Conv2d(
-            in_planes, out_planes, kernel_size=3, stride=stride, padding=0, bias=False
-        )
+        self.conv_0: nn.Module = nn.Conv2d(in_planes, out_planes, kernel_size=3, stride=stride, padding=0, bias=False)
         self.batchnorm_1: nn.Module = nn.BatchNorm2d(out_planes, momentum=bn_momentum)
         self.relu_1: nn.Module = activation_fn()
-        self.conv_1: nn.Module = nn.Conv2d(
-            out_planes, out_planes, kernel_size=3, stride=1, padding=1, bias=False
-        )
+        self.conv_1: nn.Module = nn.Conv2d(out_planes, out_planes, kernel_size=3, stride=1, padding=1, bias=False)
         self.has_shortcut: bool = in_planes != out_planes
         if self.has_shortcut:
-            self.shortcut: Optional[nn.Module] = nn.Conv2d(
+            self.shortcut: nn.Module | None = nn.Conv2d(
                 in_planes,
                 out_planes,
                 kernel_size=1,
@@ -112,7 +106,7 @@ class _Block(nn.Module):
                 bias=False,
             )
         else:
-            self.shortcut: Optional[nn.Module] = None
+            self.shortcut: nn.Module | None = None
         self._stride: int = stride
 
     def forward(self, x: torch.Tensor) -> torch.Tensor:
@@ -188,9 +182,7 @@ class _PreActBlock(nn.Module):
         )
         self.batchnorm_1: nn.Module = nn.BatchNorm2d(out_planes, momentum=bn_momentum)
         self.relu_1: nn.Module = activation_fn()
-        self.conv_2d_2: nn.Module = nn.Conv2d(
-            out_planes, out_planes, kernel_size=3, stride=1, padding=1, bias=False
-        )
+        self.conv_2d_2: nn.Module = nn.Conv2d(out_planes, out_planes, kernel_size=3, stride=1, padding=1, bias=False)
         self.has_shortcut: bool = stride != 1 or in_planes != out_planes
         if self.has_shortcut:
             self.shortcut: nn.Module = nn.Conv2d(
@@ -231,19 +223,20 @@ class WideResNet(nn.Module):
         depth: int = 28,
         width: int = 10,
         activation_fn: nn.Module = nn.SiLU,
-        mean: Union[Tuple[float, ...], float] = CIFAR10_MEAN,
-        std: Union[Tuple[float, ...], float] = CIFAR10_STD,
+        mean: tuple[float, ...] | float = CIFAR10_MEAN,
+        std: tuple[float, ...] | float = CIFAR10_STD,
         padding: int = 0,
         num_input_channels: int = 3,
         bn_momentum: float = 0.1,
+        autopool: bool = False,
     ) -> None:
         super().__init__()
         self.mean: torch.Tensor = torch.tensor(mean).view(num_input_channels, 1, 1)
         self.std: torch.Tensor = torch.tensor(std).view(num_input_channels, 1, 1)
-        self.mean_cuda: Optional[torch.Tensor] = None
-        self.std_cuda: Optional[torch.Tensor] = None
+        self.mean_cuda: torch.Tensor | None = None
+        self.std_cuda: torch.Tensor | None = None
         self.padding: int = padding
-        num_channels: List[int, int, int, int] = [
+        num_channels: list[int] = [
             16,
             16 * width,
             32 * width,
@@ -288,12 +281,11 @@ class WideResNet(nn.Module):
                 bn_momentum=bn_momentum,
             ),
         )
-        self.batchnorm: nn.Module = nn.BatchNorm2d(
-            num_channels[3], momentum=bn_momentum
-        )
+        self.batchnorm: nn.Module = nn.BatchNorm2d(num_channels[3], momentum=bn_momentum)
         self.relu: nn.Module = activation_fn()
         self.logits: nn.Module = nn.Linear(num_channels[3], num_classes)
         self.num_channels: int = num_channels[3]
+        self.pooling: nn.Module = nn.AdaptiveAvgPool2d((1, 1)) if autopool else nn.AvgPool2d(8)
 
     def forward(self, x: torch.Tensor) -> torch.Tensor:
         x: torch.Tensor = _auto_pad_to_same(self, x)
@@ -301,7 +293,7 @@ class WideResNet(nn.Module):
         out: torch.Tensor = self.init_conv(out)
         out: torch.Tensor = self.layer(out)
         out: torch.Tensor = self.relu(self.batchnorm(out))
-        out: torch.Tensor = F.avg_pool2d(out, 8)
+        out: torch.Tensor = self.pooling(out)
         out: torch.Tensor = out.view(-1, self.num_channels)
         return self.logits(out)
 
@@ -315,19 +307,20 @@ class PreActResNet(nn.Module):
         depth: int = 18,
         width: int = 0,  # Used to make the constructor consistent.
         activation_fn: nn.Module = nn.SiLU,
-        mean: Union[Tuple[float, ...], float] = CIFAR10_MEAN,
-        std: Union[Tuple[float, ...], float] = CIFAR10_STD,
+        mean: tuple[float, ...] | float = CIFAR10_MEAN,
+        std: tuple[float, ...] | float = CIFAR10_STD,
         padding: int = 0,
         num_input_channels: int = 3,
         bn_momentum: float = 0.1,
+        autopool: bool = False,
     ) -> None:
         super().__init__()
         if width != 0:
             raise ValueError("Unsupported `width = 0`.")
         self.mean: torch.Tensor = torch.tensor(mean).view(num_input_channels, 1, 1)
         self.std: torch.Tensor = torch.tensor(std).view(num_input_channels, 1, 1)
-        self.mean_cuda: Optional[torch.Tensor] = None
-        self.std_cuda: Optional[torch.Tensor] = None
+        self.mean_cuda: torch.Tensor | None = None
+        self.std_cuda: torch.Tensor | None = None
         self.padding: int = padding
         self.conv_2d: nn.Module = nn.Conv2d(
             in_channels=num_input_channels,
@@ -338,9 +331,9 @@ class PreActResNet(nn.Module):
             bias=False,
         )
         if depth == 18:
-            num_blocks: Tuple[int, int, int, int] = (2, 2, 2, 2)
+            num_blocks: tuple[int, int, int, int] = (2, 2, 2, 2)
         elif depth == 34:
-            num_blocks: Tuple[int, int, int, int] = (3, 4, 6, 3)
+            num_blocks: tuple[int, int, int, int] = (3, 4, 6, 3)
         else:
             raise ValueError("Unsupported `depth != 18` or `depth != 34`.")
         self.layer_0: nn.Module = self._make_layer(
@@ -375,13 +368,13 @@ class PreActResNet(nn.Module):
             activation_fn=activation_fn,
             bn_momentum=bn_momentum,
         )
-        self.batchnorm: nn.Module = nn.BatchNorm2d(
-            num_features=512, momentum=bn_momentum
-        )
+        self.batchnorm: nn.Module = nn.BatchNorm2d(num_features=512, momentum=bn_momentum)
         self.relu: nn.Module = activation_fn()
         self.logits: nn.Module = nn.Linear(in_features=512, out_features=num_classes)
+        self.pooling: nn.Module = nn.AdaptiveAvgPool2d((1, 1)) if autopool else nn.AvgPool2d(4)
 
-    def _make_layer(  # Do not make static.
+    # noinspection PyMethodMayBeStatic
+    def _make_layer(
         self,
         in_planes: int,
         out_planes: int,
@@ -412,6 +405,6 @@ class PreActResNet(nn.Module):
         out: torch.Tensor = self.layer_2(out)
         out: torch.Tensor = self.layer_3(out)
         out: torch.Tensor = self.relu(self.batchnorm(out))
-        out: torch.Tensor = F.avg_pool2d(out, 4)
+        out: torch.Tensor = self.pooling(out)
         out: torch.Tensor = out.view(out.size(0), -1)
         return self.logits(out)
