@@ -2,8 +2,8 @@
 # -*- coding: utf-8 -*-
 # ==============================================================================
 #
-# Copyright (c) 2020-* Emanuele Ballarin <emanuele@ballarin.cc>
-#                      All Rights Reserved.
+# Copyright (c) 2020-2025 Emanuele Ballarin <emanuele@ballarin.cc>
+#                         All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,12 +20,14 @@
 # ==============================================================================
 # SPDX-License-Identifier: Apache-2.0
 #
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import math
 
 import torch
 from torch import nn
 
-__all__ = ["ConvolutionalFlattenLayer"]
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ["ConvolutionalFlattenLayer"]
 
 
 class ConvolutionalFlattenLayer(nn.Module):
@@ -44,13 +46,10 @@ class ConvolutionalFlattenLayer(nn.Module):
     ) -> None:
         super(ConvolutionalFlattenLayer, self).__init__()
         channels_out: int = math.ceil(
-            (channels_in * height * width)
-            / ((height - detail_size + 1) * (width - detail_size + 1))
+            (channels_in * height * width) / ((height - detail_size + 1) * (width - detail_size + 1))
         )
 
-        self._output_numel: int = (
-            channels_out * (height - detail_size + 1) * (width - detail_size + 1)
-        )
+        self._output_numel: int = channels_out * (height - detail_size + 1) * (width - detail_size + 1)
 
         self.actually_flatten: bool = actually_flatten
         self.conv: nn.Module = nn.Conv2d(
