@@ -2,9 +2,9 @@
 # -*- coding: utf-8 -*-
 # ==============================================================================
 #
-# Copyright (c) 2021-* Emanuele Ballarin <emanuele@ballarin.cc>
-#                      All Rights Reserved.
-#                      [maintainance, adaptation, extension]
+# Copyright (c) 2021-2025 Emanuele Ballarin <emanuele@ballarin.cc>
+#                         All Rights Reserved.
+#                         [maintainance, adaptation, extension]
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,29 +28,29 @@
 # ==============================================================================
 # SPDX-License-Identifier: MIT
 # SPDX-License-Identifier: Apache-2.0
-# IMPORTS
-# Tensors and NN-related
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 from torch import nn
 from torch import Tensor
 
 from .functional import field_transform as ffield_transform
 
 
-__all__ = ["FieldTransform"]
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ["FieldTransform"]
 
 
 class FieldTransform(nn.Module):
     def __init__(
         self,
-        pre_sum: float = 0.0,
-        mult_div: float = 1.0,
-        post_sum: float = 0.0,
+        pre_sum: float | Tensor = 0.0,
+        mult_div: float | Tensor = 1.0,
+        post_sum: float | Tensor = 0.0,
         div_not_mul: bool = False,
     ):
         super().__init__()
-        self.pre_sum: float = pre_sum
-        self.mult_div: float = mult_div
-        self.post_sum: float = post_sum
+        self.pre_sum: float | Tensor = pre_sum
+        self.mult_div: float | Tensor = mult_div
+        self.post_sum: float | Tensor = post_sum
         self.div_not_mul: bool = div_not_mul
 
     def forward(self, x_input: Tensor) -> Tensor:
