@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # ──────────────────────────────────────────────────────────────────────────────
 #
-#  Copyright (c) 2020-2024 Emanuele Ballarin <emanuele@ballarin.cc>
+#  Copyright (c) 2020-2025 Emanuele Ballarin <emanuele@ballarin.cc>
 #  Released under the terms of the MIT License
 #  (see: https://url.ballarin.cc/mitlicense)
 #
@@ -11,7 +11,8 @@
 # SPDX-License-Identifier: MIT
 #
 # ──────────────────────────────────────────────────────────────────────────────
-# Imports (specific)
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
+from .inner_functional import batched_outer
 from .inner_functional import bisided_thresholding
 from .inner_functional import cummatmul
 from .inner_functional import field_transform
@@ -22,7 +23,8 @@ from .inner_functional import serf
 from .inner_functional import serlu
 from .inner_functional import silhouette_score
 from .inner_functional import smelu
+from .inner_functional import telu
 from .inner_functional import tensor_replicate
 
-# Deletions (from .)
+# ──────────────────────────────────────────────────────────────────────────────
 del inner_functional
