@@ -2,9 +2,9 @@
 # -*- coding: utf-8 -*-
 # ==============================================================================
 #
-# Copyright (c) 2021-* Emanuele Ballarin <emanuele@ballarin.cc>
-#                      All Rights Reserved.
-#                      [maintainance, adaptation, extension]
+# Copyright (c) 2021-2025 Emanuele Ballarin <emanuele@ballarin.cc>
+#                         All Rights Reserved.
+#                         [maintainance, adaptation, extension]
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,29 +28,29 @@
 # ==============================================================================
 # SPDX-License-Identifier: MIT
 # SPDX-License-Identifier: Apache-2.0
-# IMPORTS
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import math
-from typing import List
-from typing import Optional
-from typing import Tuple
-from typing import Union
 
 import torch
 import torch.nn.functional as F
+from thrmt.core import batched_outer
 from torch import Tensor
 
-__all__ = [
-    "field_transform",
-    "mish",
-    "serlu",
-    "smelu",
-    "serf",
-    "oldtranspose",
-    "silhouette_score",
-    "cummatmul",
-    "tensor_replicate",
-    "logit_to_prob",
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = [
     "bisided_thresholding",
+    "cummatmul",
+    "field_transform",
+    "logit_to_prob",
+    "mish",
+    "oldtranspose",
+    "serf",
+    "serlu",
+    "silhouette_score",
+    "smelu",
+    "telu",
+    "tensor_replicate",
+    "batched_outer",
 ]
 
 
@@ -59,23 +59,19 @@ __all__ = [
 
 def field_transform(
     x_input: Tensor,
-    pre_sum: Union[float, Tensor] = 0.0,
-    mult_div: Union[float, Tensor] = 1.0,
-    post_sum: Union[float, Tensor] = 0.0,
+    pre_sum: float | Tensor = 0.0,
+    mult_div: float | Tensor = 1.0,
+    post_sum: float | Tensor = 0.0,
     div_not_mul: bool = False,
 ) -> Tensor:
     if div_not_mul:
         return torch.add(
-            input=torch.div(
-                input=torch.add(input=x_input, other=pre_sum), other=mult_div
-            ),
+            input=torch.div(input=torch.add(input=x_input, other=pre_sum), other=mult_div),
             other=post_sum,
         )
     else:
         return torch.add(
-            input=torch.mul(
-                input=torch.add(input=x_input, other=pre_sum), other=mult_div
-            ),
+            input=torch.mul(input=torch.add(input=x_input, other=pre_sum), other=mult_div),
             other=post_sum,
         )
 
@@ -122,6 +118,12 @@ def serf(x: Tensor) -> Tensor:
     return torch.erf(x / math.sqrt(2.0))  # type: ignore
 
 
+@torch.jit.script
+def telu(x: Tensor) -> Tensor:
+    """Applies the TeLU function, element-wise."""
+    return x * torch.tanh(torch.exp(x))
+
+
 def oldtranspose(x: Tensor) -> Tensor:
     """
     Transpose a tensor along all dimensions, emulating x.T.
@@ -135,43 +137,31 @@ def oldtranspose(x: Tensor) -> Tensor:
     return x.permute(*torch.arange(x.ndim - 1, -1, -1))
 
 
-def silhouette_score(feats: Tensor, labels: Tensor) -> Union[float, Tensor]:  # NOSONAR
+def silhouette_score(feats: Tensor, labels: Tensor) -> float | Tensor:  # NOSONAR
     if feats.shape[0] != labels.shape[0]:
-        raise ValueError(
-            f"`feats` (shape {feats.shape}) and `labels` (shape {labels.shape}) must have same length"
-        )
+        raise ValueError(f"`feats` (shape {feats.shape}) and `labels` (shape {labels.shape}) must have same length")
     device, dtype = feats.device, feats.dtype
-    unique_labels: Union[Tensor, Tuple[Tensor, ...]] = torch.unique(labels)
+    unique_labels: Tensor | tuple[Tensor, ...] = torch.unique(labels)  # NOSONAR
     num_samples: int = feats.shape[0]
     if not (1 < len(unique_labels) < num_samples):
         raise ValueError("The number of unique `labels` must be ∈ (1, `num_samples`)")
-    scores: List[Tensor] = []
+    scores: list[Tensor] = []
     for l_label in unique_labels:
         curr_cluster: Tensor = feats[labels == l_label]
         num_elements: int = len(curr_cluster)
         if num_elements > 1:
             intra_cluster_dists: Tensor = torch.cdist(curr_cluster, curr_cluster)
-            mean_intra_dists: Tensor = torch.sum(intra_cluster_dists, dim=1) / (
-                num_elements - 1
-            )
-            dists_to_other_clusters: List[Tensor] = []
+            mean_intra_dists: Tensor = torch.sum(intra_cluster_dists, dim=1) / (num_elements - 1)
+            dists_to_other_clusters: list[Tensor] = []
             for other_l in unique_labels:
                 if other_l != l_label:
                     other_cluster: Tensor = feats[labels == other_l]
-                    inter_cluster_dists: Tensor = torch.cdist(
-                        curr_cluster, other_cluster
-                    )
-                    mean_inter_dists: Tensor = torch.sum(inter_cluster_dists, dim=1) / (
-                        len(other_cluster)
-                    )
+                    inter_cluster_dists: Tensor = torch.cdist(curr_cluster, other_cluster)
+                    mean_inter_dists: Tensor = torch.sum(inter_cluster_dists, dim=1) / (len(other_cluster))
                     dists_to_other_clusters.append(mean_inter_dists)
-            dists_to_other_clusters_t: Tensor = torch.stack(
-                dists_to_other_clusters, dim=1
-            )
+            dists_to_other_clusters_t: Tensor = torch.stack(dists_to_other_clusters, dim=1)
             min_dists: Tensor = torch.min(dists_to_other_clusters_t, dim=1)[0]
-            curr_scores: Tensor = (min_dists - mean_intra_dists) / (
-                torch.maximum(min_dists, mean_intra_dists)
-            )
+            curr_scores: Tensor = (min_dists - mean_intra_dists) / (torch.maximum(min_dists, mean_intra_dists))
         else:
             curr_scores: Tensor = torch.tensor([0], device=device, dtype=dtype)
 
@@ -185,11 +175,9 @@ def silhouette_score(feats: Tensor, labels: Tensor) -> Union[float, Tensor]:  #
     return torch.mean(scores_t)
 
 
-def cummatmul(
-    input_list: Union[List[Tensor], Tensor], tensorize: Optional[bool] = None
-) -> Union[List[Tensor], Tensor]:
+def cummatmul(input_list: list[Tensor] | Tensor, tensorize: bool | None = None) -> list[Tensor] | Tensor:
     tensorize = isinstance(input_list, Tensor) if tensorize is None else tensorize
-    cmm_list: List[Tensor] = [input_list[0]]
+    cmm_list: list[Tensor] = [input_list[0]]
     mat: Tensor
     for mat in input_list[1:]:
         cmm_list.append(torch.matmul(cmm_list[-1], mat))
@@ -210,9 +198,11 @@ def logit_to_prob(logit: Tensor) -> Tensor:
 def bisided_thresholding(x: Tensor, thresh_ile: float) -> Tensor:
     lq = min(thresh_ile, 1 - thresh_ile)
     return torch.where(
-        x > torch.quantile(x, 1 - lq),
+        x > torch.quantile(x, 1 - lq, dim=None),
         torch.ones_like(x),
         torch.where(
-            x < torch.quantile(x, lq), -torch.ones_like(x), torch.zeros_like(x)
+            x < torch.quantile(x, lq, dim=None),
+            -torch.ones_like(x),
+            torch.zeros_like(x),
         ),
     )
