@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # ==============================================================================
 #
-# Copyright 2023-* Emanuele Ballarin <emanuele@ballarin.cc>
+# Copyright 2025 Emanuele Ballarin <emanuele@ballarin.cc>
 # All Rights Reserved. Unless otherwise explicitly stated.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -21,15 +21,13 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 #
-# Imports
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import warnings
 from collections.abc import Callable
 from collections.abc import Iterable
 from collections.abc import Sequence
 from typing import Any
-from typing import Optional
 from typing import TypeVar
-from typing import Union
 
 import torch as th
 from functorch import combine_state_for_ensemble
@@ -37,7 +35,8 @@ from functorch import vmap
 from torch import nn as thnn
 from torch.optim.swa_utils import AveragedModel
 
-__all__ = ["NNEnsemble"]
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ["NNEnsemble"]
 
 
 # Custom types
@@ -46,8 +45,8 @@ T = TypeVar("T", bound="NNEnsemble")
 
 # Functions
 def _tensor_no_op(
-    x: Union[th.Tensor, Sequence[th.Tensor]]
-) -> Union[th.Tensor, Sequence[th.Tensor]]:
+    x: th.Tensor | Sequence[th.Tensor],
+) -> th.Tensor | Sequence[th.Tensor]:
     return x
 
 
@@ -61,8 +60,7 @@ def _warn_empty_models() -> None:
 
 def _warn_swa_aggregation() -> None:
     warnings.warn(
-        "swa_ensemble is True, but aggregation is not None."
-        "Aggregation will be ignored.",
+        "swa_ensemble is True, but aggregation is not None.Aggregation will be ignored.",
         UserWarning,
     )
 
@@ -71,10 +69,8 @@ def _warn_swa_aggregation() -> None:
 class NNEnsemble(thnn.Module):
     def __init__(
         self: T,
-        models: Optional[Iterable[thnn.Module]] = None,
-        aggregation: Optional[
-            Union[thnn.Module, Callable[[th.Tensor], th.Tensor]]
-        ] = None,
+        models: Iterable[thnn.Module] | None = None,
+        aggregation: thnn.Module | Callable[[th.Tensor], th.Tensor] | None = None,
         swa_ensemble: bool = False,
     ) -> None:
         super().__init__()
@@ -102,14 +98,14 @@ class NNEnsemble(thnn.Module):
             _warn_swa_aggregation()
 
         # Members required by SWA only
-        self.swa_model: Optional[AveragedModel] = None
-        self.bn_loader: Optional[Iterable] = None
+        self.swa_model: AveragedModel | None = None
+        self.bn_loader: Iterable | None = None
 
         # Members required by "pure" ensembles only
-        self.vfmodel: Optional[Any] = None
-        self.vparams: Optional[Any] = None
-        self.vbuffers: Optional[Any] = None
-        self.vensemble: Optional[Any] = None
+        self.vfmodel: Any | None = None
+        self.vparams: Any | None = None
+        self.vbuffers: Any | None = None
+        self.vensemble: Any | None = None
 
         # Test/Train safeguards
         self.notify_train_eval_changes_is_armed: bool = False
@@ -130,8 +126,7 @@ class NNEnsemble(thnn.Module):
             th.optim.swa_utils.update_bn(self.bn_loader, self.swa_model)
         else:
             warnings.warn(
-                "No batch normalization loader has been provided."
-                "Batch normalization statistics will not be updated.",
+                "No batch normalization loader has been provided.Batch normalization statistics will not be updated.",
                 UserWarning,
             )
 
@@ -143,9 +138,7 @@ class NNEnsemble(thnn.Module):
     def _update_vensemble_nofail(self: T) -> None:
         if not self.models:
             return
-        self.vfmodel, self.vparams, self.vbuffers = combine_state_for_ensemble(
-            self.models
-        )
+        self.vfmodel, self.vparams, self.vbuffers = combine_state_for_ensemble(self.models)
         self.vensemble = vmap(func=self.vfmodel, in_dims=(0, 0, None))
 
     # Internal methods for both SWA and "pure" ensembles
@@ -157,22 +150,18 @@ class NNEnsemble(thnn.Module):
 
     def _check_ensemble_is_usable(self: T, force_both: bool = False) -> None:
         if (self.swa_ensemble or force_both) and self.swa_model is None:
-            raise RuntimeError(
-                "No SWA model has been created. Did you forget to call update_state()?"
-            )
+            raise RuntimeError("No SWA model has been created. Did you forget to call update_state()?")
         if ((not self.swa_ensemble) or force_both) and self.vensemble is None:
-            raise RuntimeError(
-                "No ensemble has been created. Did you forget to call update_state()?"
-            )
+            raise RuntimeError("No ensemble has been created. Did you forget to call update_state()?")
 
     # Public methods
     def reset_state(self: T) -> T:
         self.models: thnn.ModuleList = thnn.ModuleList([])
-        self.swa_model: Optional[AveragedModel] = None
-        self.vfmodel: Optional[Any] = None
-        self.vparams: Optional[Any] = None
-        self.vbuffers: Optional[Any] = None
-        self.vensemble: Optional[Any] = None
+        self.swa_model: AveragedModel | None = None
+        self.vfmodel: Any | None = None
+        self.vparams: Any | None = None
+        self.vbuffers: Any | None = None
+        self.vensemble: Any | None = None
         return self
 
     def update_state(self: T, force_both: bool = False) -> T:
@@ -205,7 +194,7 @@ class NNEnsemble(thnn.Module):
 
     def set_aggregation(
         self,
-        aggregation: Union[thnn.Module, Callable[[th.Tensor], th.Tensor], None],
+        aggregation: thnn.Module | Callable[[th.Tensor], th.Tensor] | None,
     ):
         if aggregation is None:
             self.aggregation: Callable[[th.Tensor], th.Tensor] = _tensor_no_op
@@ -227,16 +216,12 @@ class NNEnsemble(thnn.Module):
             )
         self.bn_loader: Iterable = bn_loader
 
-    def notify_train_eval_changes(
-        self: T, armed: bool = True, hardened: bool = False
-    ) -> T:
+    def notify_train_eval_changes(self: T, armed: bool = True, hardened: bool = False) -> T:
         self.notify_train_eval_changes_is_armed: bool = armed
         self.notify_train_eval_changes_is_hardened: bool = hardened
         return self
 
-    def train(
-        self: T, mode: bool = True, override_safetynet: bool = False
-    ) -> thnn.Module:
+    def train(self: T, mode: bool = True, override_safetynet: bool = False) -> thnn.Module:
         if self.notify_train_eval_changes_is_armed and self.training != mode:
             if self.notify_train_eval_changes_is_hardened:
                 raise RuntimeError(
@@ -244,8 +229,7 @@ class NNEnsemble(thnn.Module):
                     "but denied since the model has been hardened."
                 )
             warnings.warn(
-                f"Change of training mode from {self.training} to {mode} detected."
-                "Allowing it.",
+                f"Change of training mode from {self.training} to {mode} detected.Allowing it.",
                 UserWarning,
             )
         if self.swa_ensemble and mode:
