@@ -2,8 +2,8 @@
 # -*- coding: utf-8 -*-
 # ==============================================================================
 #
-# Copyright (c) 2020-* Emanuele Ballarin <emanuele@ballarin.cc>
-#                      All Rights Reserved.
+# Copyright (c) 2020-2025 Emanuele Ballarin <emanuele@ballarin.cc>
+#                         All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,9 +19,11 @@
 #
 # ==============================================================================
 # SPDX-License-Identifier: Apache-2.0
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 from torch import nn
 
-__all__ = [
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = [
     "ReshapeLayer",
     "FlatChannelize2DLayer",
 ]
@@ -40,5 +42,6 @@ class FlatChannelize2DLayer(nn.Module):
     def __init__(self):
         super().__init__()
 
-    def forward(self, x):  # Do not make static!
+    # noinspection PyMethodMayBeStatic
+    def forward(self, x):
         return x.reshape(*x.shape, 1, 1)
