@@ -2,8 +2,8 @@
 # -*- coding: utf-8 -*-
 # ==============================================================================
 #
-# Copyright (c) 2021-* Emanuele Ballarin <emanuele@ballarin.cc>
-#                      All Rights Reserved.
+# Copyright (c) 2021-2025 Emanuele Ballarin <emanuele@ballarin.cc>
+#                         All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,13 +19,14 @@
 #
 # ==============================================================================
 # SPDX-License-Identifier: Apache-2.0
-# IMPORTS
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import torch
 from torch import Tensor
 
 from .functional import serf as fserf
 
-__all__ = ["ScaledERF"]
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ["ScaledERF"]
 
 
 class ScaledERF(torch.nn.Module):
@@ -34,5 +35,6 @@ class ScaledERF(torch.nn.Module):
     def __init__(self) -> None:
         super(ScaledERF, self).__init__()
 
-    def forward(self, x: Tensor) -> Tensor:  # Do not make static!
+    # noinspection PyMethodMayBeStatic
+    def forward(self, x: Tensor) -> Tensor:
         return fserf(x)
