@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # ==============================================================================
 #
-# Copyright (c) 2024 Emanuele Ballarin <emanuele@ballarin.cc>
+# Copyright (c) 2025 Emanuele Ballarin <emanuele@ballarin.cc>
 #                      All Rights Reserved.
 #                      [maintainance, adaptation, extension]
 #
@@ -28,25 +28,32 @@
 #
 # ==============================================================================
 # SPDX-License-Identifier: Apache-2.0
-from typing import List
-from typing import TypeVar
-
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import torch
 import torch.nn as nn
 from torch import Tensor
 
-SinLUModule = TypeVar("SinLUModule", bound="SinLU")
-
-__all__: List[str] = [
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = [
     "SinLU",
 ]
 
 
 class SinLU(nn.Module):
-    def __init__(self: SinLUModule) -> None:
-        super(SinLU, self).__init__()
-        self.a: Tensor = nn.Parameter(torch.ones(1))
-        self.b: Tensor = nn.Parameter(torch.ones(1))
+    def __init__(
+        self,
+        a: int | float = 1,
+        b: int | float = 1,
+        device: torch.device | None = None,
+        dtype: torch.dtype | None = None,
+    ) -> None:
+        factory_kwargs: dict[str, torch.device | None | torch.dtype | None] = {
+            "device": device,
+            "dtype": dtype,
+        }
+        super().__init__()
+        self.a: Tensor = nn.Parameter(torch.ones(1, **factory_kwargs) * a)
+        self.b: Tensor = nn.Parameter(torch.ones(1, **factory_kwargs) * b)
 
-    def forward(self: SinLUModule, x: Tensor) -> Tensor:
+    def forward(self, x: Tensor) -> Tensor:
         return torch.sigmoid(x) * (x + self.a * torch.sin(self.b * x))
