@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # ──────────────────────────────────────────────────────────────────────────────
 #
-#  Copyright (c) 2020-2024 Emanuele Ballarin <emanuele@ballarin.cc>
+#  Copyright (c) 2020-2025 Emanuele Ballarin <emanuele@ballarin.cc>
 #  Released under the terms of the MIT License
 #  (see: https://url.ballarin.cc/mitlicense)
 #
@@ -11,14 +11,15 @@
 # SPDX-License-Identifier: MIT
 #
 # ──────────────────────────────────────────────────────────────────────────────
-# IMPORTS
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 from torch import nn
 from torch import Tensor
 
 from .functional import smelu as fsmelu
 
 
-__all__ = ["SmeLU"]
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ["SmeLU"]
 
 
 class SmeLU(nn.Module):
