@@ -10,23 +10,21 @@
 #
 # ==============================================================================
 #
-# Copyright (c) 2020-* Emanuele Ballarin <emanuele@ballarin.cc>
-#                      All Rights Reserved.
-#                      [maintainance, adaptation, extension]
+# Copyright (c) 2020-2025 Emanuele Ballarin <emanuele@ballarin.cc>
+#                         All Rights Reserved.
+#                         [maintainance, adaptation, extension]
 #
 # ==============================================================================
 # SPDX-License-Identifier: MIT
 # SPDX-License-Identifier: Apache-2.0
-from typing import List
-from typing import Optional
-from typing import Union
-
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import numpy as np
 import torch.nn.utils as thutils
 
-__all__ = ["AutoClipper"]
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ["AutoClipper"]
 
-realnum = Union[float, int]
+realnum = float | int
 
 
 def _get_grad_norm(model) -> realnum:
@@ -40,11 +38,11 @@ def _get_grad_norm(model) -> realnum:
 
 
 class AutoClipper:
-    def __init__(self, queue_size: Optional[realnum] = None) -> None:
+    def __init__(self, queue_size: realnum | None = None) -> None:
         if queue_size is not None:
             queue_size: int = int(queue_size)
-        self.queue_size: Optional[int] = queue_size
-        self.queue_list: List[float] = []
+        self.queue_size: int | None = queue_size
+        self.queue_list: list[float] = []
 
     def grad_consider(self, model) -> None:
         if self.queue_size is not None:
@@ -54,14 +52,12 @@ class AutoClipper:
 
     def grad_autoclip_(self, model, clip_percentile) -> None:
         self.grad_consider(model)
-        thutils.clip_grad_norm_(
-            model.parameters(), np.percentile(self.queue_list, clip_percentile)
-        )
+        thutils.clip_grad_norm_(model.parameters(), np.percentile(self.queue_list, clip_percentile))
 
     def reset(self) -> None:
         self.queue_list = []
 
-    def resize(self, queue_size: Optional[realnum] = None) -> None:
+    def resize(self, queue_size: realnum | None = None) -> None:
         if queue_size is not None:
             queue_size: int = int(queue_size)
         self.queue_size = queue_size
