@@ -9,9 +9,9 @@
 #
 # Copyright (c) 2022-* Garena Online Private Limited. All Rights Reserved.
 #
-# Copyright (c) 2020-* Emanuele Ballarin <emanuele@ballarin.cc>
-#                      All Rights Reserved.
-#                      [maintainance, adaptation, extension]
+# Copyright (c) 2020-2025 Emanuele Ballarin <emanuele@ballarin.cc>
+#                         All Rights Reserved.
+#                         [maintainance, adaptation, extension]
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,12 +28,14 @@
 # ==============================================================================
 #
 # SPDX-License-Identifier: Apache-2.0
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import math
 
 import torch
 from torch.optim.optimizer import Optimizer
 
-__all__ = ["Adan"]
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ["Adan"]
 
 
 class Adan(Optimizer):
@@ -46,7 +48,7 @@ class Adan(Optimizer):
     Arguments:
         params (iterable): iterable of parameters to optimize or dicts defining parameter groups.
         lr (float, optional): learning rate. (default: 1e-3)
-        betas (Tuple[float, float, flot], optional): coefficients used for computing
+        betas (tuple[float, float, flot], optional): coefficients used for computing
             running averages of gradient and its norm. (default: (0.98, 0.92, 0.99))
         eps (float, optional): term added to the denominator to improve
             numerical stability. (default: 1e-8)
@@ -177,12 +179,8 @@ class Adan(Optimizer):
                 exp_avg_diff.mul_(beta2).add_(diff, alpha=1 - beta2)  # diff_t
                 exp_avg_sq.mul_(beta3).addcmul_(update, update, value=1 - beta3)  # n_t
 
-                denom = (exp_avg_sq.sqrt() / math.sqrt(bias_correction3)).add_(
-                    group["eps"]
-                )
-                update = (
-                    exp_avg / bias_correction1 + beta2 * exp_avg_diff / bias_correction2
-                ).div_(denom)
+                denom = (exp_avg_sq.sqrt() / math.sqrt(bias_correction3)).add_(group["eps"])
+                update = (exp_avg / bias_correction1 + beta2 * exp_avg_diff / bias_correction2).div_(denom)
 
                 if group["no_prox"]:
                     p.data.mul_(1 - group["lr"] * group["weight_decay"])
