@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # ==============================================================================
 #
-# Copyright 2024 Emanuele Ballarin <emanuele@ballarin.cc>
+# Copyright 2025 Emanuele Ballarin <emanuele@ballarin.cc>
 # All Rights Reserved. Unless otherwise explicitly stated.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -21,30 +21,34 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 #
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 from collections.abc import Callable
 from collections.abc import Iterable
 from math import ceil
-from typing import List
-from typing import Tuple
-from typing import Union
 
 import torch as th
 import torch.optim
 from torch import Tensor
+from torch.optim import Adam
 
+from .adopt import ADOPT
 from .lookahead import Lookahead
 from .radam import RAdam
 
-__all__ = [
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = [
     "ralah_optim",
+    "alah_optim",
     "wfneal",
     "tricyc1c",
     "epochwise_onecycle",
     "onecycle_lincos",
     "onecycle_linlin",
     "onecycle_linlin_updown",
+    "warmed_up_annealer",
     "warmed_up_linneal",
     "make_beta_scheduler",
+    "lahdopt",
 ]
 
 
@@ -52,16 +56,16 @@ __all__ = [
 
 
 def ralah_optim(
-    parameters: Union[Iterable[Tensor], Iterable[dict]],
+    parameters: Iterable[Tensor] | Iterable[dict],
     radam_lr: float = 1e-3,
     la_steps: int = 5,
     la_alpha: float = 0.8,
-    radam_betas: Tuple[float, float] = (0.9, 0.999),
+    radam_betas: tuple[float, float] = (0.9, 0.999),
     radam_eps: float = 1e-8,
     radam_wd: float = 0.0,
     radam_degenerate_to_sgd: bool = True,
     la_pullback_momentum: str = "none",
-):
+) -> Lookahead:
     """RAdam + Lookahead optimizer"""
     return Lookahead(
         RAdam(
@@ -78,6 +82,64 @@ def ralah_optim(
     )
 
 
+def alah_optim(
+    parameters: Iterable[Tensor] | Iterable[dict],
+    adam_lr: float = 1e-3,
+    la_steps: int = 5,
+    la_alpha: float = 0.8,
+    adam_betas: tuple[float, float] = (0.9, 0.999),
+    adam_eps: float = 1e-8,
+    adam_wd: float = 0.0,
+    la_pullback_momentum: str = "none",
+) -> Lookahead:
+    """Adam + Lookahead optimizer"""
+    return Lookahead(
+        Adam(
+            params=parameters,
+            lr=adam_lr,
+            betas=adam_betas,
+            eps=adam_eps,
+            weight_decay=adam_wd,
+        ),
+        la_steps=la_steps,
+        la_alpha=la_alpha,
+        pullback_momentum=la_pullback_momentum,
+    )
+
+
+def lahdopt(
+    parameters: Iterable[Tensor] | Iterable[dict],
+    ad_lr: float | Tensor = 1e-3,
+    la_steps: int = 5,
+    la_alpha: float = 0.8,
+    ad_betas: tuple[float, float] = (0.9, 0.9999),
+    ad_eps: float = 1e-6,
+    ad_clip_exp: float | None = 0.25,
+    ad_weight_decay: float = 0.0,
+    ad_decouple: bool = False,
+    ad_caution: bool = False,
+    ad_foreach: bool | None = False,
+    la_pullback_momentum: str = "none",
+) -> Lookahead:
+    """ADOPT + Lookahead optimizer"""
+    return Lookahead(
+        ADOPT(
+            params=parameters,
+            lr=ad_lr,
+            betas=ad_betas,
+            eps=ad_eps,
+            clip_exp=ad_clip_exp,
+            weight_decay=ad_weight_decay,
+            decouple=ad_decouple,
+            caution=ad_caution,
+            foreach=ad_foreach,
+        ),
+        la_steps=la_steps,
+        la_alpha=la_alpha,
+        pullback_momentum=la_pullback_momentum,
+    )
+
+
 # ==============================================================================
 
 
@@ -86,8 +148,7 @@ def wfneal(
     lr: float,
     epochs: int,
     magic_fraction: float = 0.56,
-    verbose: bool = False,
-) -> Tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler, int]:
+) -> tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler, int]:
     # Durations
     steady_epochs: int = int((epochs - 2) * magic_fraction)
     anneal_epochs: int = epochs - 2 - steady_epochs
@@ -106,21 +167,18 @@ def wfneal(
         end_factor=1.0,
         total_iters=2,
         last_epoch=-1,
-        verbose=verbose,
     )
     steady_scheduler = th.optim.lr_scheduler.ConstantLR(
         optimizer=optim,
         factor=1.0,
         total_iters=steady_epochs,
         last_epoch=-1,
-        verbose=verbose,
     )
     anneal_scheduler = th.optim.lr_scheduler.CosineAnnealingLR(
         optimizer=optim,
         T_max=anneal_epochs,
         eta_min=lr * 1e-4,
         last_epoch=-1,
-        verbose=verbose,
     )
 
     # Prepare scheduler
@@ -129,7 +187,6 @@ def wfneal(
         schedulers=[warmup_scheduler, steady_scheduler, anneal_scheduler],
         milestones=[2, 2 + steady_epochs],
         last_epoch=-1,
-        verbose=verbose,
     )
 
     # Return
@@ -142,7 +199,7 @@ def tricyc1c(
     max_lr: float,
     up_frac: float,
     total_steps: int,
-) -> Tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler]:
+) -> tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler]:
     """One-cycle, cyclical (triangular) learning rate scheduler."""
     # Compute durations
     up_steps = int(up_frac * total_steps)
@@ -173,8 +230,7 @@ def epochwise_onecycle(
     final_lr: float,
     up_frac: float,
     total_steps: int,
-    verbose: bool = False,
-) -> Tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler]:
+) -> tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler]:
     """Epochwise OneCycleLR learning rate scheduler."""
 
     # Prepare optim
@@ -193,7 +249,6 @@ def epochwise_onecycle(
         div_factor=max_lr / init_lr,
         final_div_factor=init_lr / final_lr,
         three_phase=False,
-        verbose=verbose,
     )
 
     # Return
@@ -207,8 +262,7 @@ def onecycle_lincos(
     final_lr: float,
     up_frac: float,
     total_steps: int,
-    verbose: bool = False,
-) -> Tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler]:
+) -> tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler]:
     """Epochwise OneCycleLR learning rate scheduler, with linear warmup and cosine annealing."""
 
     # Compute constants
@@ -227,14 +281,12 @@ def onecycle_lincos(
         end_factor=1.0,
         total_iters=warmup_steps,
         last_epoch=-1,
-        verbose=verbose,
     )
     anneal_scheduler = th.optim.lr_scheduler.CosineAnnealingLR(
         optimizer=optim,
         T_max=anneal_steps,
         eta_min=final_lr,
         last_epoch=-1,
-        verbose=verbose,
     )
 
     # Prepare scheduler
@@ -243,7 +295,6 @@ def onecycle_lincos(
         schedulers=[warmup_scheduler, anneal_scheduler],
         milestones=[warmup_steps],
         last_epoch=-1,
-        verbose=verbose,
     )
 
     # Return
@@ -257,8 +308,7 @@ def onecycle_linlin(
     final_lr: float,
     up_frac: float,
     total_steps: int,
-    verbose: bool = False,
-) -> Tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler]:
+) -> tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler]:
     """Epochwise OneCycleLR learning rate scheduler, with linear warmup and linear annealing."""
 
     # Compute constants
@@ -276,7 +326,6 @@ def onecycle_linlin(
         end_factor=1.0,
         total_iters=warmup_steps,
         last_epoch=-1,
-        verbose=verbose,
     )
     anneal_scheduler = th.optim.lr_scheduler.LinearLR(
         optimizer=optim,
@@ -284,7 +333,6 @@ def onecycle_linlin(
         end_factor=final_lr / max_lr,
         total_iters=anneal_steps,
         last_epoch=-1,
-        verbose=verbose,
     )
 
     # Prepare scheduler
@@ -293,7 +341,6 @@ def onecycle_linlin(
         schedulers=[warmup_scheduler, anneal_scheduler],
         milestones=[warmup_steps],
         last_epoch=-1,
-        verbose=verbose,
     )
 
     # Return
@@ -307,8 +354,7 @@ def onecycle_linlin_updown(
     final_lr: float,
     up_steps: int,
     down_steps: int,
-    verbose: bool = False,
-) -> Tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler]:
+) -> tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LRScheduler]:
     """
     Epochwise OneCycleLR learning rate scheduler, with linear warmup and linear annealing.
     Up/Down steps parameterization.
@@ -322,11 +368,10 @@ def onecycle_linlin_updown(
         final_lr=final_lr,
         up_frac=up_frac,
         total_steps=total_steps,
-        verbose=verbose,
     )
 
 
-def warmed_up_linneal(
+def warmed_up_annealer(
     optim: torch.optim.Optimizer,
     init_lr: float,
     steady_lr: float,
@@ -334,12 +379,13 @@ def warmed_up_linneal(
     warmup_epochs: int,
     steady_epochs: int,
     anneal_epochs: int,
+    cos_annealing: bool = False,
 ):
     # Prepare optim
     for grp in optim.param_groups:
         grp["lr"] = steady_lr
 
-    milestones: List[int] = [
+    milestones: list[int] = [
         mwue := max(3, warmup_epochs),
         mwue + max(1, steady_epochs),
     ]
@@ -358,13 +404,21 @@ def warmed_up_linneal(
         total_iters=milestones[1],
         last_epoch=-1,
     )
-    anneal_scheduler = torch.optim.lr_scheduler.LinearLR(
-        optimizer=optim,
-        start_factor=1.0,
-        end_factor=final_lr / steady_lr,
-        total_iters=max(3, anneal_epochs),
-        last_epoch=-1,
-    )
+    if not cos_annealing:
+        anneal_scheduler = torch.optim.lr_scheduler.LinearLR(
+            optimizer=optim,
+            start_factor=1.0,
+            end_factor=final_lr / steady_lr,
+            total_iters=max(3, anneal_epochs),
+            last_epoch=-1,
+        )
+    else:
+        anneal_scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(
+            optimizer=optim,
+            T_max=max(3, anneal_epochs),
+            eta_min=final_lr,
+            last_epoch=-1,
+        )
 
     # Prepare scheduler
     sched = torch.optim.lr_scheduler.SequentialLR(
@@ -378,9 +432,11 @@ def warmed_up_linneal(
     return optim, sched
 
 
-def make_beta_scheduler(
-    target: float, lag_ratio: float, wu_ratio: float
-) -> Callable[[int, int], float]:
+# Legacy alias
+warmed_up_linneal = warmed_up_annealer
+
+
+def make_beta_scheduler(target: float, lag_ratio: float, wu_ratio: float) -> Callable[[int, int], float]:
     def beta_scheduler(step: int, total: int) -> float:
         if step < (lag_steps := ceil(total * lag_ratio)):
             return 0.0
