@@ -17,7 +17,7 @@
 #                         license text: https://github.com/huggingface/pytorch-image-models/blob/main/LICENSE]
 #
 #
-# Copyright (c) 2024      Emanuele Ballarin <emanuele@ballarin.cc> (minor edits)
+# Copyright (c) 2025      Emanuele Ballarin <emanuele@ballarin.cc> (minor edits)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 #
@@ -34,12 +34,14 @@
 #
 # ──────────────────────────────────────────────────────────────────────────────
 #
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import math
 
 import torch
 from torch.optim import Optimizer
 
-__all__ = ["Lamb"]
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ["Lamb"]
 
 
 class Lamb(Optimizer):
@@ -51,7 +53,7 @@ class Lamb(Optimizer):
     Arguments:
         params (iterable): iterable of parameters to optimize or dicts defining parameter groups.
         lr (float, optional): learning rate. (default: 1e-3)
-        betas (Tuple[float, float], optional): coefficients used for computing
+        betas (tuple[float, float], optional): coefficients used for computing
             running averages of gradient and its norm. (default: (0.9, 0.999))
         eps (float, optional): term added to the denominator to improve
             numerical stability. (default: 1e-8)
@@ -108,9 +110,7 @@ class Lamb(Optimizer):
                 loss = closure()
 
         device = self.param_groups[0]["params"][0].device
-        one_tensor = torch.tensor(
-            1.0, device=device
-        )  # because torch.where doesn't handle scalars correctly
+        one_tensor = torch.tensor(1.0, device=device)  # because torch.where doesn't handle scalars correctly
         global_grad_norm = torch.zeros(1, device=device)
         for group in self.param_groups:
             for p in group["params"]:
@@ -118,9 +118,7 @@ class Lamb(Optimizer):
                     continue
                 grad = p.grad
                 if grad.is_sparse:
-                    raise RuntimeError(
-                        "Lamb does not support sparse gradients, consider SparseAdam instad."
-                    )
+                    raise RuntimeError("Lamb does not support sparse gradients, consider SparseAdam instad.")
                 global_grad_norm.add_(grad.pow(2).sum())
 
         global_grad_norm = torch.sqrt(global_grad_norm)
@@ -172,9 +170,7 @@ class Lamb(Optimizer):
                 exp_avg.mul_(beta1).add_(grad, alpha=beta3)  # m_t
                 exp_avg_sq.mul_(beta2).addcmul_(grad, grad, value=1 - beta2)  # v_t
 
-                denom = (exp_avg_sq.sqrt() / math.sqrt(bias_correction2)).add_(
-                    group["eps"]
-                )
+                denom = (exp_avg_sq.sqrt() / math.sqrt(bias_correction2)).add_(group["eps"])
                 update = (exp_avg / bias_correction1).div_(denom)
 
                 weight_decay = group["weight_decay"]
