@@ -10,19 +10,22 @@
 #
 # ==============================================================================
 #
-# Copyright (c) 2020-* Emanuele Ballarin <emanuele@ballarin.cc>
-#                      All Rights Reserved.
-#                      [maintainance, adaptation, extension]
+# Copyright (c) 2020-2025 Emanuele Ballarin <emanuele@ballarin.cc>
+#                         All Rights Reserved.
+#                         [maintainance, adaptation, extension]
 #
 # ==============================================================================
 # SPDX-License-Identifier: MIT
 # SPDX-License-Identifier: Apache-2.0
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
+import contextlib as cxl
 from collections import defaultdict
 
 import torch
 from torch.optim.optimizer import Optimizer
 
-__all__ = ["Lookahead"]
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ["Lookahead"]
 
 
 class Lookahead(Optimizer):
@@ -31,6 +34,7 @@ class Lookahead(Optimizer):
     Lookahead Optimizer: https://arxiv.org/abs/1907.08610
     """
 
+    # noinspection PyMissingConstructor
     def __init__(self, optimizer, la_steps=5, la_alpha=0.8, pullback_momentum="none"):
         """optimizer: inner optimizer
         la_steps (int): number of lookahead steps
@@ -94,6 +98,22 @@ class Lookahead(Optimizer):
                 p.data.copy_(param_state["backup_params"])
                 del param_state["backup_params"]
 
+    @cxl.contextmanager
+    def slow_weights(self):
+        self._backup_and_load_cache()
+        try:
+            yield
+        finally:
+            self._clear_and_load_backup()
+
+    @cxl.contextmanager
+    def fast_weights(self):
+        self._clear_and_load_backup()
+        try:
+            yield
+        finally:
+            self._backup_and_load_cache()
+
     @property
     def param_groups(self):
         return self.optimizer.param_groups
@@ -119,17 +139,11 @@ class Lookahead(Optimizer):
                     param_state["cached_params"].copy_(p.data)
                     if self.pullback_momentum == "pullback":
                         internal_momentum = self.optimizer.state[p]["momentum_buffer"]
-                        self.optimizer.state[p][
-                            "momentum_buffer"
-                        ] = internal_momentum.mul_(self.la_alpha).add_(
+                        self.optimizer.state[p]["momentum_buffer"] = internal_momentum.mul_(self.la_alpha).add_(
                             1.0 - self.la_alpha, param_state["cached_mom"]
                         )
-                        param_state["cached_mom"] = self.optimizer.state[p][
-                            "momentum_buffer"
-                        ]
+                        param_state["cached_mom"] = self.optimizer.state[p]["momentum_buffer"]
                     elif self.pullback_momentum == "reset":
-                        self.optimizer.state[p]["momentum_buffer"] = torch.zeros_like(
-                            p.data
-                        )
+                        self.optimizer.state[p]["momentum_buffer"] = torch.zeros_like(p.data)
 
         return loss
