@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # =============================================================================
-# Copyright (c) 2024   Jiangtao Zhang and Shunyu Liu and Jie Song and
+# Copyright (c) 2025   Jiangtao Zhang and Shunyu Liu and Jie Song and
 #                      Tongtian Zhu and Zhengqi Xu and Mingli Song
 #                      [original work: https://arxiv.org/abs/2306.07684]
 #                      All Rights Reserved. Apache-2.0 Licensed.
@@ -18,25 +18,25 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-# Copyright (c) 2024 Emanuele Ballarin <emanuele@ballarin.cc> [minor edits]
+# Copyright (c) 2025 Emanuele Ballarin <emanuele@ballarin.cc> [minor edits]
 #                      All Rights Reserved. MIT Licensed.
 # ==============================================================================
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import copy
-from typing import List
-from typing import Optional
 
 import torch
 from torch import Tensor
 from torch.optim.optimizer import Optimizer
 from torch.optim.optimizer import required  # type: ignore
 
-__all__ = ("Lookaround",)
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ("Lookaround",)
 
 
 def _sgd(
-    params: List[Tensor],
-    d_p_list: List[Tensor],
-    momentum_buffer_list: List[Optional[Tensor]],
+    params: list[Tensor],
+    d_p_list: list[Tensor],
+    momentum_buffer_list: list[Tensor | None],
     weight_decay: float,
     momentum: float,
     lr: float,
@@ -50,7 +50,7 @@ def _sgd(
         if weight_decay != 0:
             d_p: Tensor = d_p.add(param, alpha=weight_decay)
         if momentum != 0:
-            buf: Optional[Tensor] = momentum_buffer_list[i]
+            buf: Tensor | None = momentum_buffer_list[i]
             if buf is None:
                 buf: Tensor = torch.clone(d_p).detach()
                 momentum_buffer_list[i]: Tensor = buf
@@ -145,9 +145,7 @@ class Lookaround(Optimizer):
 
             m_str = "momentum_buffer_" + str(head)
 
-            r = (self.step_n % (group["frequence"] * group["head_num"])) % group[
-                "head_num"
-            ]
+            r = (self.step_n % (group["frequence"] * group["head_num"])) % group["head_num"]
 
             for p in group["params"]:
                 if p.grad is not None:
@@ -173,24 +171,15 @@ class Lookaround(Optimizer):
                 self.state[p][m_str] = momentum_buffer
             for i, p in enumerate(group["params"]):
                 self.net_head[r][i].data = group["params"][i].data
-            if (self.step_n % (group["frequence"] * group["head_num"])) + 1 == (
-                group["frequence"] * group["head_num"]
-            ):
+            if (self.step_n % (group["frequence"] * group["head_num"])) + 1 == (group["frequence"] * group["head_num"]):
                 for i, p in enumerate(group["params"]):
-                    self.net_head[0][i][:] = (
-                        1 / group["head_num"] * self.net_head[0][i][:]
-                    )
+                    self.net_head[0][i][:] = 1 / group["head_num"] * self.net_head[0][i][:]
                     for j in range(1, group["head_num"]):
-                        self.net_head[0][i][:] = (
-                            self.net_head[0][i][:]
-                            + 1 / group["head_num"] * self.net_head[j][i][:]
-                        )
+                        self.net_head[0][i][:] = self.net_head[0][i][:] + 1 / group["head_num"] * self.net_head[j][i][:]
                     for j in range(1, group["head_num"]):
                         self.net_head[j][i][:] = self.net_head[0][i][:]
             for i, p in enumerate(group["params"]):
-                group["params"][i].data = self.net_head[(r + 1) % group["head_num"]][
-                    i
-                ].data
+                group["params"][i].data = self.net_head[(r + 1) % group["head_num"]][i].data
 
         self.step_n += 1
         return loss
