@@ -16,7 +16,7 @@
 #                      [orig. code: https://github.com/huggingface/pytorch-image-models ;
 #                      license text: https://github.com/huggingface/pytorch-image-models/blob/main/LICENSE]
 #
-# Copyright (c) 2024 Emanuele Ballarin <emanuele@ballarin.cc> (minor edits)
+# Copyright (c) 2025 Emanuele Ballarin <emanuele@ballarin.cc> (minor edits)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 #
@@ -33,6 +33,7 @@
 #
 # ──────────────────────────────────────────────────────────────────────────────
 #
+# ~~ Imports ~~ ────────────────────────────────────────────────────────────────
 import math
 
 import torch
@@ -40,7 +41,8 @@ import torch.nn.functional as F
 from torch.optim.optimizer import Optimizer
 from torch.optim.optimizer import required
 
-__all__ = ["SGDP", "AdamP"]
+# ~~ Exports ~~ ────────────────────────────────────────────────────────────────
+__all__: list[str] = ["SGDP", "AdamP"]
 
 
 def _channel_view(x: torch.Tensor) -> torch.Tensor:
@@ -128,19 +130,11 @@ class SGDP(Optimizer):
                 # Projection
                 wd_ratio = 1.0
                 if len(p.shape) > 1:
-                    d_p, wd_ratio = _projection(
-                        p, grad, d_p, group["delta"], group["wd_ratio"], group["eps"]
-                    )
+                    d_p, wd_ratio = _projection(p, grad, d_p, group["delta"], group["wd_ratio"], group["eps"])
 
                 # Weight decay
                 if weight_decay != 0:
-                    p.mul_(
-                        1.0
-                        - group["lr"]
-                        * group["weight_decay"]
-                        * wd_ratio
-                        / (1 - momentum)
-                    )
+                    p.mul_(1.0 - group["lr"] * group["weight_decay"] * wd_ratio / (1 - momentum))
 
                 # Step
                 p.add_(d_p, alpha=-group["lr"])
@@ -205,9 +199,7 @@ class AdamP(Optimizer):
                 exp_avg.mul_(beta1).add_(grad, alpha=1 - beta1)
                 exp_avg_sq.mul_(beta2).addcmul_(grad, grad, value=1 - beta2)
 
-                denom = (exp_avg_sq.sqrt() / math.sqrt(bias_correction2)).add_(
-                    group["eps"]
-                )
+                denom = (exp_avg_sq.sqrt() / math.sqrt(bias_correction2)).add_(group["eps"])
                 step_size = group["lr"] / bias_correction1
 
                 if nesterov:
